/*
 * Name:     CActor.cpp
 *
 *
 * Class:    CActor
 * Header:   CActor.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CActor.h"
#include "CDocument.h"
#include "CTimeline.h"

/*
 * Name :        CActor::CActor()
 * Description : Actor class constructor
 */
CActor::CActor()
{
}

/*
 * Name :        CActor::CActor()
 * Description : Constructor for an actor.
 */
CActor::CActor(const std::wstring &name) : mName(name)
{
    mEnabled = true;
    mPosition = wxPoint(0, 0);
    mDefaultPos = wxPoint(0, 0);
    mClickable = true;
    mDocument = NULL;
    mRoot = NULL;
}

/*
 * Name :        CActor::~CActor()
 * Description : Actor class destructor.
 */
CActor::~CActor()
{
	// Move along now, nothing to see here
	delete mRoot;
}

/*
 * Name :        CActor::AddDrawable()
 * Description : Add a drawable to the actor. We add in order we will draw.
 * Parameters :  drawable - The drawable to add.
 * Returns :     none
 */
void CActor::AddDrawable(CDrawable *drawable)
{
	mDrawablesInOrder.push_back(drawable);
	drawable->SetActor(this);
}

/*
 * Name :        CActor::Draw()
 * Description : Draw the actor.
 * Parameters :  dc - The device context to draw on .
 * Returns :     none
 */
void CActor::Draw(wxDC *dc)
{
    // Don't draw if not enabled
    if(!mEnabled)
        return;

    // This takes care of determining the absolute placement
    // of all of the child drawables. We have to determine this
    // in tree order, which may not be the order we draw.
    if(mRoot)
        mRoot->Place(mPosition, 0);

    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->Draw(dc);
    }
}


/*
 * Name :        CActor::HitTest()
 * Description : Determine if the mouse clicked on this actor.
 * Parameters :  pos - The x,y position to test.
 * Returns :     Pointer to any drawable that was hit or NULL if none.
 */
CDrawable *CActor::HitTest(wxPoint pos)
{
    // If not enabled or not clickable, we indicate no hit.
    if(!mClickable || !mEnabled)
        return NULL;

    // Since this list is in drawing order, we realy want to know the last thing drawn
    // under the mouse, since it will be on top. So, we reverse iterate over the list.
    for(std::list<CDrawable *>::reverse_iterator d=mDrawablesInOrder.rbegin(); d!=mDrawablesInOrder.rend(); d++)
    {
        CDrawable *drawable = *d;
        if(drawable->HitTest(pos))
            return *d;
    }

    return NULL;
}

/*
 * Name :        CActor::SetDocument()
 * Description : Set the document link for this actor, telling it what
 *               document to use. Also tells all child drawables what the
 *               timeline is.
 * Parameters :  doc - The document we are using.
 * Returns :
 */
void CActor::SetDocument(CDocument *doc)
{
    mDocument = doc;

    // Set the timeline for all drawables. This links the channels to
    // the timeline system.
    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->SetTimeline(mDocument->GetTimeline());
    }
    mDocument->GetTimeline()->AddChannel(&mChannel);
    mDefaultPos = mPosition;
}

/*
 * Name :        CActor::SetKeyframe()
 * Description : Set a keyframe on an actor.
 * Parameters :  none
 * Returns :     none
 */
void CActor::SetKeyframe()
{
    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->SetKeyframe();
    }
    mChannel.SetKeyframe(mPosition);
}

/*
 * Name :        CActor::GetKeyframe()
 * Description : Get a keyframe for an actor.
 * Parameters :  none
 * Returns :     none
 */
void CActor::GetKeyframe()
{
    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->GetKeyframe();
    }
    mPosition = mChannel.GetPos(mPosition);
}

/*
 * Name :        CActor::ClearKeyframe()
 * Description : Remove a keyframe from an actor.
 * Parameters :  none
 * Returns :     none
 */
void CActor::ClearKeyframe()
{
    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->ClearKeyframe();
    }
    mChannel.ClearKeyframe();
}

/*
 * Name :        CActor::XmlSave()
 * Description : Saves animation for an actor to XML
 * Parameters :  The pointer to the keyframe, and where it is
 * Returns :     none
 */
void CActor::XmlSave(wxXmlNode* keyframe, int here, bool first)
{
	CTimeline *timeline = mDocument->GetTimeline();
	double temp = timeline->GetCurrentTime();

	// Convert spaces to underscores for XML
	std::wstring name = GetName();
	std::wstring spc(L" ");
	std::wstring und(L"_");
	for (unsigned int i = 0; i < name.length(); i++)
	{
		if (name[i] == spc)
		{
			name[i] =und[0];
		}
	}

	// Create a new node for an actor
	wxXmlNode *actor = new wxXmlNode(wxXML_ELEMENT_NODE, name);
	timeline->SetCurrentTime((double(mChannel.GetKeyframe(here)->GetFrame()) / double(timeline->GetFrameRate())));
	if (first)
		keyframe->AddProperty(L"time", wxString::Format(L"%f", timeline->GetCurrentTime()));

	// Add the offset for an actor
	//wxXmlNode *offset = new wxXmlNode(wxXML_ELEMENT_NODE, L"offset");
	wxPoint tempPos = mChannel.GetPos(mPosition);
	actor->AddProperty(L"x", wxString::Format(L"%d", tempPos.x));
	actor->AddProperty(L"y", wxString::Format(L"%d", tempPos.y));
	// Don't  need animation channel to phase out actors, default to all active
	//actor->AddProperty(L"enable", wxString::Format(L"%d", mEnabled));
	//actor->AddChild(offset);

	// Add the angle/position of all drawable children
    for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
    {
        CDrawable *drawable = *d;
        drawable->XmlSaveDrawable(actor);
    }

	timeline->SetCurrentTime(temp);
	keyframe->AddChild(actor);
}

/*
 * Name :        CActor::ClearChannels()
 * Description : Dumps the contents of all channels in the actor
 * Parameters :  none
 * Returns :     none
 */
void CActor::ClearChannels()
{
	mChannel.EmptyChannel();
	for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
	{
		CDrawable *drawable = *d;
		drawable->ClearChannels();
	}
}

/*
 * Name :        CActor::SetDefault()
 * Description : Sets position/rotation of object to first set
 * Parameters :  none
 * Returns :     none
 */
void CActor::SetDefault()
{
	mPosition = mDefaultPos;
	GetRoot()->SetRotation(mDefaultRot);
	for(std::list<CDrawable *>::iterator d=mDrawablesInOrder.begin(); d!=mDrawablesInOrder.end(); d++)
	{
		CDrawable *drawable = *d;
		drawable->SetDefault();
	}
}

/*
 * Name :        CActor::SetHDrawable()
 * Description : Sets position of drawables based on child/parent relation
 * Parameters :  none
 * Returns :     none
 */
void CActor::SetHDrawable(CDrawable *source)
{
	CIteratorPtr<CDrawable *> iters = GetDrawables();
	mDrawablesInHeirarchy.push_back(source);
	for(iters->First();  !iters->IsDone();  iters->Next())
	{
		CDrawable *child = iters->Current();
		if (child->GetParent() == source)
		{
			SetHDrawable(child);
		}
	}
}
