/*
 * Name:     CDrawable.cpp
 *
 *
 * Class:    CDrawable
 * Header:   CDrawable.h
 * Created : Feb 26, 2010
 * Author :  Kenji Aono
 */

#include "wx/prec.h"
#include "CActor.h"
#include "CDrawable.h"
#include <cmath>

/*
 * Name :        CDrawable::CDrawable()
 * Description : Drawable class constructor
 */
CDrawable::CDrawable()
{
}

/*
 * Name :        CDrawable::CDrawable()
 * Description : Constructor for a drawable.
 * Parameters :  name - the name of the drawable.
 * Returns :     none
 */
CDrawable::CDrawable(const std::wstring &name) : mName(name)
{
    mPosition = wxPoint(0, 0);
    mRotation = 0;
    mPlacedPosition = wxPoint(0, 0);
    mPlacedR = 0;
    mActor = NULL;
    mParent = NULL;
    mEnabled = true;
}

/*
 * Name :        CDrawable::!CDrawable()
 * Description : Drawable class destructor
 */
CDrawable::~CDrawable()
{
	for(std::list<CDrawable *>::iterator c=mChildren.begin(); c!=mChildren.end(); c++)
	{
		delete *c;
	}

	mChildren.clear();
}

/*
 * Name :        CDrawable::Place()
 * Description : Place the drawables at the correct location. This works heirarchically
 *               from top item down.
 * Parameters :  offset - An x,y offset
 *               rotation - A rotation
 * Returns :     none
 */
void CDrawable::Place(wxPoint offset, double rotate)
{
    // Combine the transformation we are given with the transformation
    // for this object.
    mPlacedPosition = offset + RotatePoint(mPosition, rotate);
    mPlacedR = mRotation + rotate;

    // Update our children
    for(std::list<CDrawable *>::iterator d=mChildren.begin(); d!=mChildren.end(); d++)
    {
        (*d)->Place(mPlacedPosition, mPlacedR);
    }
}

/*
 * Name :        CDrawable::AddChild()
 * Description : Add a child to his drawable's composite pattern.
 * Parameters :  child - The child to add.
 * Returns :     none
 */
void CDrawable::AddChild(CDrawable *child)
{
	mChildren.push_back(child);
	child->SetParent(this);
}

/*
 * Name :        CDrawable::AddSChild()
 * Description : Add a child to his drawable's composite pattern without setting default (for CHead)
 * Parameters :  child - The child to add.
 * Returns :     none
 */
void CDrawable::AddSChild(CDrawable *child)
{
	mChildren.push_back(child);
	child->SetParent(this);
}

/*
 * Name :        CDrawable::IsMovable()
 * Description : Determine if a drawable is moveable. By default most are not movable.
 * Parameters :  none
 * Returns :     true if object is movable.
 */
bool CDrawable::IsMovable()
{
    return false;
}

/*
 * Name :        CDrawable::Move()
 * Description : Move the object relative to it's parent by a delta in screen coordinates.
 * Parameters :  delta - Distance to move.
 * Returns :     none
 */
void CDrawable::Move(wxPoint delta)
{
	// Need to account for a parent or root having a rotation, and adjust delta
	// Follow how changing an angle works, a mouse up always moves the
	// drawable 'up' if a parent's rotation is considered as zero
	double tempPR = GetParent()->GetRotation();
	double tempRR = GetActor()->GetRoot()->GetRotation();
	const double PI = 3.14159265358;
	// The total angle away from zero(up), +(clockwise) -(counter-clockwise)
	double totalRot = (tempPR + tempRR );//+ mRotation);
	totalRot = fmod(totalRot, 2*PI);
	// Calculate offset considering rotations of objects
	wxPoint newDel( ((delta.x * cos(totalRot)) + (delta.y * cos(totalRot + PI/2))),
	                ((delta.y * cos(totalRot)) + (delta.x * cos(totalRot - PI/2))));
	SetPosition(wxPoint((GetPosition().x+ newDel.x), (GetPosition().y + newDel.y)));
}

/*
 * Name :        CDrawable::RotatePoint()
 * Description : Rotate a point by a given angle.
 * Parameters :  point - The point to rotate
 *               angle - An angle in radians
 * Returns :     Rotated point
 *
 * Note:         This functions such that moving the cursor
 *               up rotates clockwise, and down counter-clockwise
 */
wxPoint CDrawable::RotatePoint(wxPoint point, double angle)
{
    double cosA = cos(angle);
    double sinA = sin(angle);

    return wxPoint(cosA * point.x + sinA * point.y,
                  -sinA * point.x + cosA * point.y);
}

/*
 * Name :        CDrawable::SetActor()
 * Description : Set the actor that uses this drawable.
 * Parameters :  actor - The actor that uses us.
 * Returns :     none
 */
void CDrawable::SetActor(CActor *actor)
{
    mActor = actor;
    mDefaultPos = mPosition;
    mDefaultRot = mRotation;
    // Set the channel name
    mChannel.SetName(actor->GetName() + L":" + mName);
}

/*
 * Name :        CDrawable::SetTimeline()
 * Description : Set the timeline for all channels.
 * Parameters :  timeline - The timeline class.
 * Returns :     none
 */
void CDrawable::SetTimeline(CTimeline *timeline)
{
    timeline->AddChannel(&mChannel);
    timeline->AddChannel(&mChannel2);
}

/*
 * Name :        CDrawable::SetKeyframe()
 * Description : Set the keyframe based on the current status.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::SetKeyframe()
{
    mChannel.SetKeyframe(mRotation);
    mChannel2.SetKeyframe(mPosition);
}

/*
 * Name :        CDrawable::GetKeyframe()
 * Description : Get the current channel from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::GetKeyframe()
{
	if (mChannel.GetNumKeyframes() > 0)
	{
		mRotation = mChannel.GetAngle();
	}
	if (mChannel2.GetNumKeyframes() > 0)
	{
		mPosition = mChannel2.GetPos(mPosition);
	}
}

/*
 * Name :        CDrawable::ClearKeyframe()
 * Description : Clear the current frame from the animation system.
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::ClearKeyframe()
{
    mChannel.ClearKeyframe();
    mChannel2.ClearKeyframe();
}

/*
 * Name :        CDrawable::ClearChannels()
 * Description : Clear the channel of all keyframes
 * Parameters :  none
 * Returns :     none
 */
void CDrawable::ClearChannels()
{
	mChannel.EmptyChannel();
	mChannel2.EmptyChannel();
}

/*
 * Name :        CDrawable::XmlSaveDrawable()
 * Description : Saves information for a drawable
 * Parameters :  The actor it belongs to
 * Returns :     none
 */
void CDrawable::XmlSaveDrawable(wxXmlNode* actor)
{
	// Convert any spaces to an underscore
	std::wstring name = GetName();
	std::wstring spc(L" ");
	std::wstring und(L"_");
	for (unsigned int i = 0; i < name.length(); i++)
	{
		if (name[i] == spc)
		{
			name[i] =und[0];
		}
	}

	// Create a node for this drawable item
	wxXmlNode *drawable = new wxXmlNode(wxXML_ELEMENT_NODE, name);

	// Add a position if applicable to save space
	if (IsMovable())
	{
		//wxXmlNode *position = new wxXmlNode(wxXML_ELEMENT_NODE, L"position");
		wxPoint tempPos = mChannel2.GetPos(mPosition);
		drawable->AddProperty(L"x", wxString::Format(L"%d", tempPos.x));
		drawable->AddProperty(L"y", wxString::Format(L"%d", tempPos.y));
		//drawable->AddChild(position);
	}

	// Add angle information (default to zero?)
	//wxXmlNode *angle = new wxXmlNode(wxXML_ELEMENT_NODE, L"angle");
	double tempAng = mChannel.GetAngle();
	// Only save two decimal points for angles since the rest are 0
	drawable->AddProperty(L"angle", wxString::Format(L"%.2f", tempAng));
	//drawable->AddChild(angle);

	actor->AddChild(drawable);
}

/*
 * Name :        CDrawable::SetPosition()
 * Description : Sets the position
 * Parameters :  The new position
 * Returns :     none
 */
void CDrawable::SetPosition(wxPoint p)
{
	mPosition = p;
}

/*
 * Name :        CDrawable::SetRotation()
 * Description : Sets the rotation
 * Parameters :  The new angle
 * Returns :     none
 */
void CDrawable::SetRotation(double r)
{
	mRotation = r;
}

