/*
 * Name:     CViewActors.cpp
 *
 *
 * Class:    CViewActors
 * Header:   CViewActors.h
 * Created : Feb 20, 2010
 * Author :  Charles B. Owen - Initial distributed version
 * Co-Author: Kenji Aono
 */

#include "wx/prec.h"
#include "CViewActors.h"
#include "CFrame.h"

BEGIN_EVENT_TABLE(CViewActors, wxScrolledWindow)
    EVT_PAINT(CViewActors::OnPaint)
    EVT_LEFT_DOWN(CViewActors::OnLeftButtonDown)
    EVT_RIGHT_DOWN(CViewActors::OnRightButtonDown)
END_EVENT_TABLE()

/*
 * Name :        CViewActors::CViewActors()
 * Description : Constructor for the actor view.
 */
CViewActors::CViewActors(CFrame *frame) :
    wxScrolledWindow(frame, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxVSCROLL | wxBORDER_SUNKEN),
    CDocumentObserver(frame->GetDocument())
{
    // This will determine the size. Only the horizontal part matters
    // The sizer will take care of the vertical part for us
    SetMinSize(wxSize(150, 100));
    SetBackgroundColour(wxColour(255, 255, 255));
    SetScrollRate(1, 1);
    mDispChild = false;
}

/*
 * Name :        CViewActors::~CViewActors()
 * Description : The view destructor.
 */
CViewActors::~CViewActors()
{
}


/*
 * Name :        CViewActors::OnPaint()
 * Description : Handle a paint request for this window.
 * Parameters :  event - The paint event
 * Returns :     none
 */
void CViewActors::OnPaint(wxPaintEvent &event)
{
    // Create a device context
    wxPaintDC dc(this);

    // Offset the device context due to any scrolling
    DoPrepareDC(dc);

    // Create the font
    wxFont font(12, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    dc.SetFont(font);

    wxCoord w, h;
    dc.GetTextExtent(L"TEST", &w, &h);
    mTextHeight = h;

    // Print out actors involved, and if applicable their parts
    mY = 0;
    mIdx = 0;
    CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
    for(iter->First();  !iter->IsDone();  iter->Next())
    {
        dc.SetTextForeground(wxColour(0, 0, 0));
        CActor *actor = iter->Current();
        dc.DrawText(actor->GetName(), 0, mY);
        mY += mTextHeight;
        if (mDispChild)
        {
			dc.SetTextForeground(wxColour(0,0,128));
			OnPaintDraw(&dc, actor->GetRoot());
        }
        mIdx -= 10;
    }

    // Setting size for the scrollbar
    SetVirtualSize(wxSize(170, mY));

    dc.SetPen(wxNullPen);
    dc.SetBrush(wxNullBrush);
    dc.SetFont(wxNullFont);
}


/*
 * Name :        CViewActors::OnLeftButtonDown()
 * Description : Handle a left mouse button press.
 * Parameters :  event - The command event
 * Returns :     none
 */
void CViewActors::OnLeftButtonDown(wxMouseEvent &event)
{
    // Get the view point start and add to the mouse position
    // to determine where we clicked in the scrolled area.
    int x, y;
    GetViewStart(&x, &y);
    x += event.m_x;
    y += event.m_y;

    int entry = y/mTextHeight + 1;
    // Clicked on valid entry
    if ((entry-1) < (mY/mTextHeight))
    {
    	int curr = 0;
    	CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
		for(iter->First();  !iter->IsDone();  iter->Next())
		{
			CActor *actor = iter->Current();
			curr++;
			if (curr == entry) {actor->TogEnabled(); break;}
			// Are we in expanded view?
	        if (mDispChild)
	        {
				CIteratorPtr<CDrawable *> iters = actor->GetRoot()->GetActor()->GetHDrawables();
				for(iters->First();  !iters->IsDone();  iters->Next())
				{
					CDrawable *drawable = iters->Current();
					curr++;
					if (curr == entry) {drawable->TogEnabled(); break;}
				}
	        }
		}
		// Inform others of the changes
		GetDocument()->UpdateObservers();
    }
}

/*
 * Name :        CViewActors::UpdateObserver()
 * Description : Force an update of this window when the document changes.
 * Parameters :  none
 * Returns :     none
 */
void CViewActors::UpdateObserver()
{
    Refresh();
}

/*
 * Name :        CViewActors::OnPaintDraw()
 * Description : Used to recursively display the parents, then children of the CActors
 * Parameters :  Device context to draw to, the pointer to the individual CDrawable
 * Returns :     none
 */
void CViewActors::OnPaintDraw(wxDC *dc, CDrawable *source)
{
	mIdx += 10;
	dc->DrawText(source->GetName(), mIdx, mY );
	mY += mTextHeight;
    CIteratorPtr<CDrawable *> iters = source->GetActor()->GetHDrawables();
    for(iters->First();  !iters->IsDone();  iters->Next())
    {
        CDrawable *child = iters->Current();
        if (child->GetParent() == source)
        {
        	OnPaintDraw(dc, child);
        	mIdx -= 10;
        }
    }
}

/*
 * Name :        CViewActors::OnRightButtonDown()
 * Description : Handle a right mouse button press, will reset a drawable to default position
 * Parameters :  event - The command event
 * Returns :     none
 */
void CViewActors::OnRightButtonDown(wxMouseEvent &event)
{
    // Get the view point start and add to the mouse position
    // to determine where we clicked in the scrolled area.
    int x, y;
    GetViewStart(&x, &y);
    x += event.m_x;
    y += event.m_y;

    int entry = y/mTextHeight + 1;
    // Clicked on valid entry
    if ((entry-1) < (mY/mTextHeight))
    {
    	int curr = 0;
    	bool done = false;
    	CIteratorPtr<CActor *> iter = GetDocument()->GetActors();
		for(iter->First();  !iter->IsDone();  iter->Next())
		{
			CActor *actor = iter->Current();
			curr++;
			if (curr == entry) {actor->SetDefault(); break;}
			// Are we in expanded view?
	        if (mDispChild)
	        {
			    CIteratorPtr<CDrawable *> iters = actor->GetHDrawables();
			    for(iters->First();  !iters->IsDone();  iters->Next())
			    {
			        CDrawable *drawable = iters->Current();
			        curr++;
					if (curr == entry) {drawable->SetDefault(); done = true; break;}
			    }
	        }
	        if (done) {break;}
		}
		// Inform others of the changes
		GetDocument()->UpdateObservers();
    }
}
