/*
 * Name:     CViewTimeline.cpp
 * This class manages the timeline window at the bottom of the screen.
 *
 * Class:    CViewTimeline
 * Header:   CViewTimeline.h
 * Created : Feb 20, 2010
 * Author :  Charles B. Owen - Initial distributed version
 * Co-Author: Kenji Aono
 */

#include "wx/prec.h"
#include "CViewTimeline.h"
#include "CFrame.h"

// The window height
const int WindowHeight = 65;

// The spacing between ticks in the timeline
const int TickSpacing = 3;

// The length of a short tick mark
const int TickShort = 10;

// The length of a long tick mark
const int TickLong = 20;

// The amount of space to leave under the tick marks
const int TickUnder = 10;

// Space to the left of the scale
const int BorderLeft = 10;

// Space to the right of the scale
const int BorderRight = 10;

BEGIN_EVENT_TABLE(CViewTimeline, wxScrolledWindow)
    EVT_PAINT(CViewTimeline::OnPaint)
	EVT_LEFT_DOWN(CViewTimeline::OnLeftButtonDown)
	EVT_LEFT_UP(CViewTimeline::OnMouseMove)
	EVT_MOTION(CViewTimeline::OnMouseMove)
END_EVENT_TABLE()

/*
 * Name :        CViewTimeline::CViewTimeline()
 * Description : Constructor for the timeline view.
 */
CViewTimeline::CViewTimeline(CFrame *frame) :
    wxScrolledWindow(frame, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxHSCROLL | wxSUNKEN_BORDER),
    CDocumentObserver(frame->GetDocument())
{
	SetMinSize(wxSize(100, WindowHeight));
	SetScrollRate(1, 0);
    mPointer.LoadFile(L"images/pointer.png");
    mMovingPointer = false;
}


/*
 * Name :        CViewTimeline::~CViewTimeline()
 * Description : Destructor for the timeline view.
 */
CViewTimeline::~CViewTimeline()
{
}

/*
 * Name :        CViewTimeline::UpdateObserver()
 * Description : Force an update of this window when the document changes.
 * Parameters :  none
 * Returns :     none
 */
void CViewTimeline::UpdateObserver()
{
    Refresh();
}

/*
 * Name :        CViewTimeline::OnPaint()
 * Description : Handle a paint request for this window.
 * Parameters :  event - The paint event
 * Returns :     none
 */
void CViewTimeline::OnPaint(wxPaintEvent &event)
{
    // Create a device context
    wxPaintDC dc(this);

    // Get the timeline
    CTimeline *timeline = GetDocument()->GetTimeline();

    // Set the scroll system correctly
    SetVirtualSize(timeline->GetNumFrames() * TickSpacing + BorderLeft + BorderRight, WindowHeight);

    // Offset the device context due to any scrolling
    DoPrepareDC(dc);

    // Get window dimensions
    int wid, hit;
    GetClientSize(&wid, &hit);

    wxPen tickPen(*wxBLACK, 1);
    dc.SetPen(tickPen);

    wxFont tickFont(8, wxFONTFAMILY_SWISS, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_NORMAL);
    dc.SetFont(tickFont);

    // Draw the tick marks
    // The number of ticks (frames) to show
    unsigned int f = timeline->GetNumFrames();
    // The frame rate, indicates when long ticks should be used
    unsigned int r = timeline->GetFrameRate();
    for (unsigned int curr = 0;curr <= f; curr++)
    {
    	int tempX = BorderLeft + (TickSpacing * curr);
    	// The test for 0 % 0 == 0 isn't liked, moved into individual if
    	if (curr == 0)
    	{
            wxString num;
            num.Printf(L"%d", 0);
            wxCoord tw, th;
            dc.GetTextExtent(num, &tw, &th);
            dc.DrawText(num, (tempX - (0.4 * tw)), (hit - TickUnder - TickLong - th));
    		dc.DrawLine(tempX, (hit - TickUnder), tempX, (hit - TickUnder - TickLong));
    		continue;
    	}
    	if (curr % r == 0)
    	{
            wxString num;
            num.Printf(L"%d", curr / timeline->GetFrameRate());
            wxCoord tw, th;
            dc.GetTextExtent(num, &tw, &th);
            dc.DrawText(num, (tempX - (0.4 * tw)), (hit - TickUnder - TickLong - th));
    		dc.DrawLine(tempX, (hit - TickUnder), tempX, (hit - TickUnder - TickLong));
    	}
    	else
    	{
    		dc.DrawLine(tempX, (hit - TickUnder), tempX, (hit - TickUnder - TickShort));
    	}
    }

    // Draw the pointer, assume center is offset 5 pixels from the right
    double t = timeline->GetCurrentTime();
    int temp = t * r * TickSpacing + 5;
   	dc.DrawBitmap(mPointer, (temp / TickSpacing * TickSpacing + 2), (hit - TickUnder - TickShort), true);

    dc.SetPen(wxNullPen);
    dc.SetBrush(wxNullBrush);
    dc.SetFont(wxNullFont);
}

/*
 * Name :        CFrame::OnLeftButtonDown
 * Description : Handles a left mouse click
 * Parameters :  The object describing the event
 * Returns :     void
 */
void CViewTimeline::OnLeftButtonDown(wxMouseEvent& event)
{
    // Get the view point start and add to the mouse position
    // to determine where we clicked in the scrolled area.
    int x, y;
    GetViewStart(&x, &y);
    x += event.m_x;
    y += event.m_y;

    // Get the timeline
    CTimeline *timeline = GetDocument()->GetTimeline();
    int pointerX = timeline->GetCurrentTime() * timeline->GetFrameRate() * TickSpacing + BorderLeft;

    // Set flag to allow moving of the pointer
    mMovingPointer = x >= pointerX - mPointer.GetWidth() / 2 && x <= pointerX + mPointer.GetWidth() / 2;
}

/*
 * Name :        CFrame::OnMouseMove
 * Description : Handles a moving the mouse
 * Parameters :  The object describing the event
 * Returns :     void
 */
void CViewTimeline::OnMouseMove(wxMouseEvent& event)
{
	if(!event.m_leftDown)
	{
		mMovingPointer = false;
	}
	if (mMovingPointer)
	{
	    // Get the view point start and add to the mouse position
	    // to determine where we clicked in the scrolled area.
	    int x, y;
	    GetViewStart(&x, &y);
	    x += event.m_x;
	    y += event.m_y;

		// Get the timeline
		CTimeline *timeline = GetDocument()->GetTimeline();
		if ( timeline->GetFrameRate() != 0)
		{
			double place = (x - BorderLeft) / TickSpacing;
			if (place >= 0 && place <= timeline->GetNumFrames())
			{
				GetDocument()->SetAnimationTime(place / timeline->GetFrameRate());
			}
		}
    }
}
